
#include <iostream>
#include <chrono>
#include <thread>
#include <opencv2/opencv.hpp>
// All vpi headers are under directory vpi/
#include <vpi/OpenCVInterop.hpp>
#include <vpi/Image.h>
#include <vpi/Stream.h>
#include <vpi/algo/ConvertImageFormat.h>
using namespace std::chrono;

void threadTask1(std::string fileName, VPIStream stream, VPIImage img, VPIImage imgGray)
{
    cv::Mat cvImage = cv::imread(fileName);
    if (cvImage.data == NULL)
    {
        std::cerr << "Can't open input image" << std::endl;
        return;
    }

    vpiImageCreateOpenCVMatWrapper(cvImage, 0, &img);
    vpiImageCreate(cvImage.cols, cvImage.rows, VPI_IMAGE_FORMAT_U8, 0, &imgGray);

    VPIStatus st = vpiSubmitConvertImageFormat(stream, VPI_BACKEND_CUDA, img, imgGray, NULL);
    if(st != VPI_SUCCESS)
    {
        std::cout << "vpiSubmitConvertImageFormat error=" << st << std::endl;
    }

    vpiStreamSync(stream);
    vpiImageDestroy(img);
}

void threadTask2(std::string fileName, VPIStream stream, VPIImage img, VPIImage imgGray)
{
    cv::Mat cvImage = cv::imread(fileName);
    if (cvImage.data == NULL)
    {
        std::cerr << "Can't open input image" << std::endl;
        return;
    }

    vpiImageCreateOpenCVMatWrapper(cvImage, 0, &img);
    vpiImageCreate(cvImage.cols, cvImage.rows, VPI_IMAGE_FORMAT_U8, 0, &imgGray);

    VPIStatus st = vpiSubmitConvertImageFormat(stream, VPI_BACKEND_CUDA, img, imgGray, NULL);
    if(st != VPI_SUCCESS)
    {
        std::cout << "vpiSubmitConvertImageFormat error=" << st << std::endl;
    }

    vpiStreamSync(stream);

    VPIImageData outData;
    vpiImageLock(imgGray, VPI_LOCK_READ, &outData);

    cv::Mat cvOut(outData.planes[0].height, outData.planes[0].width, CV_8UC1, outData.planes[0].data,
                  outData.planes[0].pitchBytes);
    imwrite("tutorial_gray_t2.jpg", cvOut);

    vpiImageUnlock(imgGray);

    vpiStreamDestroy(stream);
    vpiImageDestroy(img);
    vpiImageDestroy(imgGray);
}

/* Run command as follows:
 * $ ./vpi_fmtcvt_thread img.jpeg */
int main(int argc, char *argv[])
{
    if (argc != 2)
    {
        std::cerr << "Must pass an input image to be blurred" << std::endl;
        return 1;
    }

    VPIStream stream;
    vpiStreamCreate(0, &stream);

    VPIImage t1_image, t1_imageGray;
    VPIImage t2_image, t2_imageGray;

    std::thread vpiImageWork_t1(threadTask1, argv[1], stream, t1_image, t1_imageGray);
    std::thread vpiImageWork_t2(threadTask2, argv[1], stream, t2_image, t2_imageGray);

    vpiImageWork_t1.join();
    vpiImageWork_t2.join();

    VPIImageData outData;
    vpiImageLock(t1_imageGray, VPI_LOCK_READ, &outData);

    cv::Mat cvOut(outData.planes[0].height, outData.planes[0].width, CV_8UC1, outData.planes[0].data,
                  outData.planes[0].pitchBytes);

    imwrite("tutorial_gray_t1.jpg", cvOut);
    vpiImageUnlock(t1_imageGray);

    // Destroy all created objects.
    vpiStreamDestroy(stream);
    // vpiImageDestroy(t1_image);
    vpiImageDestroy(t1_imageGray);

    // std::cout << "t_vpiFmtCvt=" << t_vpiFmtCvt.elapsed() << " t_streamWork=" << t_streamWork.elapsed() << " t_vpiImgAccess=" << t_vpiImgAccess.elapsed() << std::endl;

    return 0;
}
